@@ -23,9 +23,8 @@
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 ;;
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; A copy of the GNU General Public License is available at
+;; http://www.r-project.org/Licenses/
 
 ;;; Code:
 
@@ -133,7 +132,7 @@
   :prefix "ess-")
 ;; Variables (not user-changeable)
 
-(defvar ess-version "13.09" ;; updated by 'make'
+(defvar ess-version "14.09" ;; updated by 'make'
   "Version of ESS currently loaded.")
 
 (defvar ess-revision nil ;; set
@@ -226,9 +225,8 @@ See also `tooltip-hide-delay' and `tooltip-delay'.
 
 (defcustom ess-R-describe-object-at-point-commands
   '(("str(%s)")
-    ("head(%s, n = 40)")
-    ("tail(%s, n = 40)")
-    ("summary(%s)"))
+    ("htsummary(%s, hlength = 20, tlength = 20)")
+    ("summary(%s, maxsum = 20)"))
   "A list of commands cycled by `ess-describe-object-at-point'.
 %s is substituted with the name at point.
 
@@ -263,7 +261,7 @@ See also `ess-blink-delay'"
   :type 'boolean)
 
 (defcustom ess-blink-delay .3
-  "Number of secons to highlight the evaluated region."
+  "Number of seconds to highlight the evaluated region."
   :group 'ess
   :type 'number)
 
@@ -288,12 +286,12 @@ See also `ess-blink-delay'"
 Currently acceptable values are `S',  `XLS', `SAS'.
 Can be changed, e.g., to `R'.  Use `setq-default' if setting it in
 .emacs (also see ess-site.el).")
-  ;; :group 'ess
-  ;; :type '(choice (const :tag "Initial" :value nil)
-  ;;                (const :tag "S"       :value "S")
-  ;;                (const :tag "XLS"     :value "XLS")
-  ;;                (const :tag "SAS"     :value "SAS")
-  ;;                (const :tag "R"       :value "R")))
+;; :group 'ess
+;; :type '(choice (const :tag "Initial" :value nil)
+;;                (const :tag "S"       :value "S")
+;;                (const :tag "XLS"     :value "XLS")
+;;                (const :tag "SAS"     :value "SAS")
+;;                (const :tag "R"       :value "R")))
 
 (make-variable-buffer-local 'ess-language)
 
@@ -472,7 +470,6 @@ to install your custom sources.
   :group 'ess-extras
   :type '(choice (const t) (const script-only) (const nil)))
 
-
 (defcustom ess-ac-R-argument-suffix " = "
   "Suffix appended by `ac-source-R' and `ac-source-R-args' to candidates."
   :group 'R
@@ -492,6 +489,10 @@ might want to set this to nil.
   :group 'ess
   :type 'boolean)
 
+
+(defvar ess-ac-sources nil
+  "Dialect specific, ESS specific list of ac-sources")
+
 (defvar ess--completing-hist nil
   "Variable to store completion history.
 Used by `ess-completion-read' command.")
@@ -648,6 +649,11 @@ nil means to use R/S indentation.")
   "Extra indentation for open braces.
 Compares with other text in same context.")
 
+(defvar ess-first-continued-statement-offset 0
+  "Extra indentation for the first new line continuing an expression.
+If you set this to non-zero value you might want to set
+`ess-continued-statement-offset' to zero.")
+
 (defvar ess-continued-statement-offset 2
   "Extra indent for lines not starting new statements.")
 
@@ -734,6 +740,7 @@ If not number, the statements are indented at open-parenthesis following
 (defvar ess-default-style-list
   (list 'DEFAULT
         (cons 'ess-indent-level '(default-value 'ess-indent-level))
+        (cons 'ess-first-continued-statement-offset '(default-value 'ess-first-continued-statement-offset))
         (cons 'ess-continued-statement-offset '(default-value 'ess-continued-statement-offset))
         (cons 'ess-brace-offset '(default-value 'ess-brace-offset))
         (cons 'ess-expression-offset '(default-value 'ess-expression-offset))
@@ -742,61 +749,76 @@ If not number, the statements are indented at open-parenthesis following
         (cons 'ess-continued-brace-offset '(default-value 'ess-continued-brace-offset))
         (cons 'ess-arg-function-offset '(default-value 'ess-arg-function-offset))
         (cons 'ess-arg-function-offset-new-line '(default-value 'ess-arg-function-offset-new-line))
-        (cons 'ess-close-brace-offset '(default-value 'ess-close-brace-offset)))
-  "Default style constructed from initial values of indentation variables.")
+        (cons 'ess-close-brace-offset '(default-value 'ess-close-brace-offset))
+        )
+  "Style constructed from initial (default) values of ESS indentation variables.")
 
 (defvar ess-style-alist
   (cons ess-default-style-list
         '((GNU (ess-indent-level . 2)
+               (ess-first-continued-statement-offset . 0)
                (ess-continued-statement-offset . 2)
                (ess-brace-offset . 0)
                (ess-arg-function-offset . 4)
                (ess-arg-function-offset-new-line . '(4))
                (ess-expression-offset . 2)
                (ess-else-offset . 0)
-               (ess-close-brace-offset . 0))
+               (ess-close-brace-offset . 0)
+               )
           (BSD (ess-indent-level . 8)
+               (ess-first-continued-statement-offset . 0)
                (ess-continued-statement-offset . 8)
                (ess-brace-offset . -8)
                (ess-arg-function-offset . 0)
                (ess-arg-function-offset-new-line . '(8))
                (ess-expression-offset . 8)
                (ess-else-offset . 0)
-               (ess-close-brace-offset . 0))
+               (ess-close-brace-offset . 0)
+               )
           (K&R (ess-indent-level . 5)
+               (ess-first-continued-statement-offset . 0)
                (ess-continued-statement-offset . 5)
                (ess-brace-offset . -5)
                (ess-arg-function-offset . 0)
                (ess-arg-function-offset-new-line . '(5))
                (ess-expression-offset . 5)
                (ess-else-offset . 0)
-               (ess-close-brace-offset . 0))
+               (ess-close-brace-offset . 0)
+               )
           (C++ (ess-indent-level . 4)
+               (ess-first-continued-statement-offset . 0)
                (ess-continued-statement-offset . 4)
                (ess-brace-offset . -4)
                (ess-arg-function-offset . 0)
                (ess-arg-function-offset-new-line . '(4))
                (ess-expression-offset . 4)
                (ess-else-offset . 0)
-               (ess-close-brace-offset . 0))
+               (ess-close-brace-offset . 0)
+               )
           ;; R added ajr 17Feb04 to match "common R" use
           (RRR (ess-indent-level . 4)
+               (ess-first-continued-statement-offset . 0)
                (ess-continued-statement-offset . 4)
                (ess-brace-offset . 0)
                (ess-arg-function-offset . 4)
                (ess-arg-function-offset-new-line . '(4))
                (ess-expression-offset . 4)
                (ess-else-offset . 0)
-               (ess-close-brace-offset . 0))
+               (ess-close-brace-offset . 0)
+               )
           ;; CLB added rmh 2Nov97 at request of Terry Therneau
           (CLB (ess-indent-level . 2)
+               (ess-first-continued-statement-offset . 0)
                (ess-continued-statement-offset . 4)
                (ess-brace-offset . 0)
                (ess-arg-function-offset . 0)
                (ess-arg-function-offset-new-line . '(2))
                (ess-expression-offset . 4)
                (ess-else-offset . 0)
-               (ess-close-brace-offset . 2))))
+               (ess-close-brace-offset . 2)
+               )
+          )
+        )
   "Predefined formatting styles for ESS code.
 Values for all groups, except OWN, are fixed.  To change the
 value of variables in the OWN group, customize the variable
@@ -806,6 +828,7 @@ value of variables in the OWN group, customize the variable
 (defun ess-add-style (key entries)
   "Add a new style to `ess-style-list', with the key KEY.
 Remove any existing entry with the same KEY before adding the new one.
+               (ess-first-continued-statement-offset . 0)
 This can be used"
   (setq ess-style-alist (assq-delete-all key ess-style-alist))
   (add-to-list 'ess-style-alist (cons key entries)))
@@ -871,8 +894,8 @@ to edit more than one object at a time, though.
  (make-temp-name \"scr.\") ; Another way to uniquify"
   ;; MM: The last 3-4 lines above suck (I don't understand them) -- FIXME --
 
-:group 'ess-edit
-:type 'string)
+  :group 'ess-edit
+  :type 'string)
 
 
 ;;*;; Hooks
@@ -963,7 +986,7 @@ If nil, ESS will try finding one from a list."
   :group 'ess-roxy
   :type 'string)
 
-(defcustom ess-roxy-tags-noparam '("export" "nord")
+(defcustom ess-roxy-tags-noparam '("export" "noRd")
   "The tags used in roxygen fields that can be used alone.  Used
 to decide highlighting and tag completion."
   :group 'ess-roxy
@@ -1680,7 +1703,7 @@ If you wish to pass arguments to a process, see e.g. `inferior-R-args'.")
 
 (defvar inferior-ess-secondary-prompt nil
   "Regular expression used by ess-mode to detect the secondary prompt.
-(This is issued by S to continue an incomplete expression).
+This is issued by S to continue an incomplete expression.
 Set to nil if language doesn't support secondary prompt.")
 ;; :group 'ess-proc
 ;; :type 'string)
@@ -1693,7 +1716,7 @@ Set to nil if language doesn't support secondary prompt.")
 
 ;; need to recognise  + + + > > >
 ;; and "+ . + " in tracebug prompt
-(defcustom inferior-S-prompt "[]a-zA-Z0-9.[]*\\([>+.] \\)*> "
+(defcustom inferior-S-prompt "[]a-zA-Z0-9.[]*\\(?:[>+.] \\)*> "
   "Regexp used in S and R inferior and transcript buffers for prompt navigation.
 Customise it to make `comint-previous-prompt' quiqly navigate to
 interesting portions of the buffer.
@@ -1876,7 +1899,7 @@ This format string should use %s to substitute an object name.")
 (setq-default inferior-ess-help-command "help(\"%s\")\n")
 
 
-(defcustom inferior-ess-r-help-command ".ess_help(\"%s\", help_type=\"text\")\n"
+(defcustom inferior-ess-r-help-command ".ess.help(\"%s\", help.type=\"text\")\n"
   "Format-string for building the R command to ask for help on an object.
 
 This format string should use %s to substitute an object name.
@@ -1993,7 +2016,7 @@ session.")
 
 (defvar ess-sp-change nil
   "Variable not used. Use (ess-process-get 'sp-for-help-changed?) instead.")
-(make-obsolete-variable 'ess-sp-change nil "ESS 12.09")
+(make-obsolete-variable 'ess-sp-change nil "ESS[12.09]")
 ;; (make-variable-buffer-local 'ess-sp-change)
 
 (defvar ess-prev-load-dir/file nil
@@ -2064,11 +2087,7 @@ dialects' alists.  Increase this, if you have a fast(er) machine."
 
 ;;; for programming, transcript, and inferior process modes.
 
-(defcustom ess-font-lock-mode
-  (if  (ess-running-emacs-version-or-newer 22 1)
-      global-font-lock-mode
-    ;; else for emacs 21.x and earlier
-    t)
+(defcustom ess-font-lock-mode global-font-lock-mode
   "Non-nil means we use font lock support for ESS buffers.
 Default is t, to use font lock support.
 If you change the value of this variable, restart Emacs for it to take effect."
@@ -2236,15 +2255,15 @@ default or not."
 
 ;;; fl-keywords R
 (defvar ess-R-fl-keyword:modifiers
-   (cons (concat "\\<" (regexp-opt ess-R-modifyiers 'enc-paren) "\\>")
-         'font-lock-constant-face)     ; modify search list or source (i.e. directives)
-   "Font-lock keyword R modifiers")
+  (cons (concat "\\<" (regexp-opt ess-R-modifyiers 'enc-paren) "\\>")
+        'font-lock-constant-face)     ; modify search list or source (i.e. directives)
+  "Font-lock keyword R modifiers")
 
 (defvar ess-R-fl-keyword:fun-defs
-   (cons ess-R-function-name-regexp
-         '(1 font-lock-function-name-face t)  ; override
-         )
-   "Font-lock keyword - function defintions for R.")
+  (cons ess-R-function-name-regexp
+        '(1 font-lock-function-name-face t)  ; override
+        )
+  "Font-lock keyword - function defintions for R.")
 
 (defvar ess-R-fl-keyword:keywords
   (cons (concat "\\<" (regexp-opt ess-R-keywords 'enc-paren) "\\>")
@@ -2396,8 +2415,8 @@ default."
 
 (defcustom ess-help-pop-to-buffer t
   "If non-nil ess-help buffers are given focus during the display.
-The default is t.
-"
+The default is t (except when `focus-follows-mouse' and
+`mouse-autoselect-window' are both t)."
   :group 'ess-help
   :type 'boolean)
 
@@ -2541,10 +2560,11 @@ Defaults to `ess-S-non-functions'."
 
 
  ; julia-mode
-(defcustom inferior-julia-program-name "julia-basic"
-  ;; the default assumes it is on the PATH ... which is typically the case after
-  ;; a "typical unix-alike installation"
-  "Path to julia-basic executable"
+(defcustom inferior-julia-program-name (if (executable-find "julia-basic")
+                                           "julia-basic"
+                                         "julia")
+  "julia' executable.
+Need to be a full path if julia executable is not in the `exec-path'"
   :group 'ess-Julia)
 
 (defvar julia-basic-offset 4
