@@ -22,9 +22,8 @@
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; A copy of the GNU General Public License is available at
+;; http://www.r-project.org/Licenses/
 
 ;;; Commentary:
 
@@ -154,7 +153,7 @@
     (define-key map "\C-c\C-k"   'ess-force-buffer-current)
     (define-key map "\C-c`"      'ess-show-traceback)
     (define-key map [(control ?c) ?~] 'ess-show-call-stack)
-    (define-key map "\C-c."      'ess-set-style); analogous to binding in C-mode
+    (define-key map "\C-c."      (lambda () (interactive) (message "ess-set-style moved to C-c C-e C-s. Sorry for the inconvenience")))
     (define-key map "{"          'ess-electric-brace)
     (define-key map "}"          'ess-electric-brace)
     (define-key map "\C-\M-q"    'ess-indent-exp)
@@ -164,6 +163,8 @@
     ;;(define-key map [delete]   'backward-delete-char-untabify)
     (define-key map "\t"         'ess-indent-or-complete)
     (define-key map "\C-c\C-q"   'ess-quit)
+    (define-key map "\M-\r"      'ess-use-this-dir)
+
     ;; smart operators; most likely will go in the future into a separate local map
     (define-key map ","          'ess-smart-comma)
 
@@ -188,22 +189,26 @@
     ;; (define-key map "\M-j"    'ess-eval-line-and-go)
     map)
   "Keymap for ess-eval functions.")
-(make-obsolete-variable 'ess-eval-map nil "ESS12.09.1")
+(make-obsolete-variable 'ess-eval-map nil "ESS[12.09.1]")
 
 (defvar ess-extra-map
   (let (ess-extra-map)
     (define-prefix-command 'ess-extra-map)
     (define-key ess-extra-map "\C-d" 'ess-dump-object-into-edit-buffer)
     (define-key ess-extra-map "d" 'ess-dump-object-into-edit-buffer)
-    (define-key ess-extra-map "\C-t" 'ess-build-tags-for-directory)
-    (define-key ess-extra-map "t" 'ess-build-tags-for-directory)
-    (define-key ess-extra-map "\C-l" 'ess-load-library)
-    (define-key ess-extra-map "l" 'ess-load-library)
+    (define-key ess-extra-map "\C-e" 'ess-execute)
+    (define-key ess-extra-map "e" 'ess-execute)
     (define-key ess-extra-map "\C-i" 'ess-install-library)
     (define-key ess-extra-map "i" 'ess-install-library)
+    (define-key ess-extra-map "\C-l" 'ess-load-library)
+    (define-key ess-extra-map "l" 'ess-load-library)
+    (define-key ess-extra-map "\C-s" 'ess-set-style)
+    (define-key ess-extra-map "s" 'ess-set-style)
+    (define-key ess-extra-map "\C-t" 'ess-build-tags-for-directory)
+    (define-key ess-extra-map "t" 'ess-build-tags-for-directory)
     (define-key ess-extra-map "\C-w" 'ess-execute-screen-options)
     (define-key ess-extra-map "w" 'ess-execute-screen-options)
-    ;; (define-key map "C-t"
+    (define-key ess-extra-map "/" 'ess-set-working-directory)
     ess-extra-map)
   "ESS extra map")
 
@@ -219,7 +224,7 @@
     ["Eval region | func | para" ess-eval-region-or-function-or-paragraph t]
     ["Eval region | func | para & step" ess-eval-region-or-function-or-paragraph-and-step t]
     ["Eval region | line" ess-eval-region-or-line-and-step t]
-    ["Enter expression" ess-execute-in-tb                 t]
+    ["Enter expression" ess-execute                 t]
     ;; sub menus
     "------"
     ("Process"
@@ -304,7 +309,7 @@
     "------"
     ("Font Lock"
      :active ess-font-lock-keywords
-     :filter ess-generate-font-lock-submenu)
+     :filter ess--generate-font-lock-submenu)
     "------"
     ["Describe"         describe-mode                   t]
     ["About editing" (ess-goto-info "Editing")  t]
@@ -389,9 +394,14 @@ Variables controlling indentation style:
     Indentation of ESS statements within surrounding block.
     The surrounding block's indentation is the indentation of the line on
     which the open-brace appears.
+ `ess-first-continued-statement-offset'
+    Extra indentation given to the first continued statement.
  `ess-continued-statement-offset'
     Extra indentation given to a substatement, such as the then-clause of an
     if or body of a while.
+ `ess-dont-align-on-assignment'
+    Whether we should align continued statements based on the position of a
+    `<-` or `=` assignment operator.
  `ess-continued-brace-offset'
     Extra indentation given to a brace that starts a substatement.
     This is in addition to ess-continued-statement-offset.
@@ -523,7 +533,7 @@ ess-mode."
      (repl "\\(<-\\)?")                 ; replacement (function)
      (Sym-0 "\\(\\sw\\|\\s_\\)")        ; symbol
      (Symb  (concat Sym-0 "+"))
-     (xSymb (concat "\\[?\\[?" Sym-0 "*")); symbol / [ / [[ / [symbol / [[symbol
+     (xSymb "[^ \t\n\"']+") ;; (concat "\\[?\\[?" Sym-0 "*")); symbol / [ / [[ / [symbol / [[symbol
      ;; FIXME: allow '%foo%' but only when quoted; don't allow [_0-9] at beg.
      (_or_  "\\)\\|\\(")                ; OR
      (space "\\(\\s-\\|\n\\)*")         ; white space
@@ -531,8 +541,7 @@ ess-mode."
      (part-1 (concat
               "\\(" ;;--------outer Either-------
               "\\(\\("          ; EITHER
-              ;; Q xSymb repl Sym-0 "*" Q  ; quote ([) (replacement) symbol quote
-              Q "[^ \t\n\"']+" Q ;; any function name between quotes
+              Q xSymb Q         ; any function name between quotes
               _or_
               "\\(^\\|[ ]\\)" Symb      ; (beginning of name) + Symb
               "\\)\\)"))        ; END EITHER OR
@@ -555,6 +564,7 @@ ess-mode."
               space "function\\s-*(" ; whitespace, function keyword, parenthesis
               ))
      )
+
   (defvar ess-R-function-pattern
     (concat part-1
             "\\s-*\\(<-\\|=\\)" ; whitespace, assign
@@ -822,7 +832,7 @@ With prefix argument, only shows the errors ESS reported."
 		   (buffer-substring (match-beginning 2) (match-end 2))))
 		 (errmess (buffer-substring (match-beginning 1) (match-end 1))))
 	    (if showerr
-		  (ess-display-temp-buffer errbuff)
+                (ess-display-temp-buffer errbuff)
 	      (if fbuffer nil
 		(setq fbuffer (find-file-noselect filename))
 		(with-current-buffer fbuffer
@@ -941,127 +951,11 @@ The default of `ess-tab-complete-in-script' is nil.  Also see
 (defun ess-indent-exp ()
   "Indent each line of the ESS grouping following point."
   (interactive)
-  (let ((indent-stack (list nil))
-        (contain-stack (list (point)))
-        (case-fold-search nil)
-        ;; restart
-        outer-loop-done innerloop-done state ostate
-        this-indent
-        last-sexp
-        last-depth
-        at-else at-brace
-        (opoint (point))
-        (next-depth 0))
-    (save-excursion
-      (forward-sexp 1))
-    (save-excursion
-      (setq outer-loop-done nil)
-      (while (and (not (eobp)) (not outer-loop-done))
-        (setq last-depth next-depth)
-        ;; Compute how depth changes over this line
-        ;; plus enough other lines to get to one that
-        ;; does not end inside a comment or string.
-        ;; Meanwhile, do appropriate indentation on comment lines.
-        (setq innerloop-done nil)
-        (while (and (not innerloop-done)
-                    (not (and (eobp) (setq outer-loop-done t))))
-          (setq ostate state)
-          (setq state (parse-partial-sexp (point) (progn (end-of-line) (point))
-                                          nil nil state))
-          (setq next-depth (car state))
-          (if (and (car (cdr (cdr state)))
-                   (>= (car (cdr (cdr state))) 0))
-              (setq last-sexp (car (cdr (cdr state)))))
-          (if (or (nth 4 ostate))
-              (ess-indent-line))
-          (if (nth 4 state)
-              (and (ess-indent-line)
-                   (setcar (nthcdr 4 state) nil)))
-          (if (or (nth 3 state))
-              (forward-line 1)
-            (setq innerloop-done t)))
-        (if (<= next-depth 0)
-            (setq outer-loop-done t))
-        (if outer-loop-done
-            nil
-          ;; If this line had ..))) (((.. in it, pop out of the levels
-          ;; that ended anywhere in this line, even if the final depth
-          ;; doesn't indicate that they ended.
-          (while (> last-depth (nth 6 state))
-            (setq indent-stack (cdr indent-stack)
-                  contain-stack (cdr contain-stack)
-                  last-depth (1- last-depth)))
-          (if (/= last-depth next-depth)
-              (setq last-sexp nil))
-          ;; Add levels for any parens that were started in this line.
-          (while (< last-depth next-depth)
-            (setq indent-stack (cons nil indent-stack)
-                  contain-stack (cons nil contain-stack)
-                  last-depth (1+ last-depth)))
-          (if (null (car contain-stack))
-              (setcar contain-stack (or (car (cdr state))
-                                        (save-excursion (forward-sexp -1)
-                                                        (point)))))
-          (forward-line 1)
-          (skip-chars-forward " \t")
-          (if (eolp)
-              nil
-            (if (and (car indent-stack)
-                     (>= (car indent-stack) 0))
-                ;; Line is on an existing nesting level.
-                ;; Lines inside parens are handled specially.
-                (if (/= (char-after (car contain-stack)) ?{)
-                    (setq this-indent (car indent-stack))
-                  ;; Line is at statement level.
-                  ;; Is it a new statement?  Is it an else?
-                  ;; Find last non-comment character before this line
-                  (save-excursion
-                    (setq at-else (looking-at "else\\W"))
-                    (setq at-brace (= (following-char) ?{))
-                    (ess-backward-to-noncomment opoint)
-                    (if (ess-continued-statement-p)
-                        ;; Preceding line did not end in comma or semi;
-                        ;; indent this line  ess-continued-statement-offset
-                        ;; more than previous.
-                        (progn
-                          (ess-backward-to-start-of-continued-exp (car contain-stack))
-                          (setq this-indent
-                                (+ ess-continued-statement-offset (current-column)
-                                   (if at-brace ess-continued-brace-offset 0))))
-                      ;; Preceding line ended in comma or semi;
-                      ;; use the standard indent for this level.
-                      (if at-else
-                          (progn (ess-backward-to-start-of-if opoint)
-                                 (setq this-indent (+ ess-else-offset
-                                                      (current-indentation))))
-                        (setq this-indent (car indent-stack))))))
-              ;; Just started a new nesting level.
-              ;; Compute the standard indent for this level.
-              (let ((val (ess-calculate-indent
-                          (if (car indent-stack)
-                              (- (car indent-stack))))))
-                (setcar indent-stack
-                        (setq this-indent val))))
-            ;; Adjust line indentation according to its contents
-            (if (= (following-char) ?})
-                ;;(setq this-indent (- this-indent ess-indent-level)))
-                (setq this-indent (+ this-indent
-                                     (- ess-close-brace-offset ess-indent-level))))
-            (if (= (following-char) ?{)
-                (setq this-indent (+ this-indent ess-brace-offset)))
-            ;; Put chosen indentation into effect.
-            (or (= (current-column) this-indent)
-                (= (following-char) ?\#)
-                (progn
-                  (delete-region (point) (progn (beginning-of-line) (point)))
-                  (indent-to this-indent)))
-            ;; Indent any comment following the text.
-            (or (looking-at comment-start-skip)
-                (if (re-search-forward comment-start-skip
-                                       (save-excursion (end-of-line)
-                                                       (point)) t)
-                    (progn (indent-for-comment) (beginning-of-line))))))))))
-;; (message "Indenting ESS expression...done")
+  (save-excursion
+    (let ((start (point))
+          (end (ignore-errors (forward-sexp 1) (point))))
+      (when end
+        (indent-region start end)))))
 
 ;;;*;;; Support functions for indentation
 
@@ -1108,7 +1002,7 @@ Return the amount the indentation changed by."
 				(not (looking-at "else\\s_")))
 			   (setq indent (save-excursion
 					  (ess-backward-to-start-of-if)
-					  (+ ess-else-offset (current-indentation)))))
+					  (+ ess-else-offset (current-column)))))
 			  ((= (following-char) ?})
 			   (setq indent
 				 (+ indent
@@ -1125,195 +1019,276 @@ Return the amount the indentation changed by."
 	;; If initial point was within line's indentation,
 	;; position after the indentation.
 	;; Else stay at same point in text.
-	(if (> (- (point-max) pos) (point))
-	    (goto-char (- (point-max) pos))))
+	(when (> (- (point-max) pos) (point))
+          (goto-char (- (point-max) pos))))
       shift-amt)))
 
+
+(defun ess-looking-at-last-open-paren-p ()
+  (looking-at "[[:blank:]]*([[:blank:]]*\\($\\|#\\)"))
+
+(defun ess-calculate-indent--closing-paren ()
+  (search-forward ")")
+  (backward-sexp)
+  (if (ess-looking-at-last-open-paren-p)
+      ;; If this line ends with "("
+      (current-indentation)
+    ;; otherwise
+    (+ (current-column) 1)))
+
+(defun ess-calculate-indent--default (&optional parse-start)
+  (let ((indent-point (point))
+        (beginning-of-defun-function nil) ;; don't call ess-beginning-of-function
+        (open-paren-in-column-0-is-defun-start t) ;; basically go to point-min (no better solution aparently)
+        (case-fold-search nil)
+        state ind
+        containing-sexp)
+    (if parse-start
+        (goto-char parse-start)
+      ;; this one is awful; with open-paren-in-column-0-is-defun-start t, it
+      ;; always goes to point-min
+      (beginning-of-defun))
+    (while (< (point) indent-point)
+      (setq parse-start (point))
+      (setq state (parse-partial-sexp (point) indent-point 0))
+      (setq containing-sexp (car (cdr state))))
+    (cond ((or (nth 3 state) (nth 4 state))
+           ;; return nil (in string) or t (in comment)
+           (nth 4 state))
+          ((and (null containing-sexp)
+                (= (following-char) ?\{))
+           0)
+          ((setq ind (ess--continued-block-statement containing-sexp))
+           (+ ind ess-indent-level))
+          ((setq ind (ess--continued-statement containing-sexp))
+           (+ ind ess-continued-statement-offset))
+          ((null containing-sexp)
+           0)
+          ;; ;; Line is at top level.  May be data or function definition
+          ;; ;; or continuation of if,for,else not folowed by {
+          ;; (beginning-of-line)
+          ;; (if (/= (following-char) ?\{)
+          ;;     (cond ((setq ind (ess--continued-block-statement containing-sexp))
+          ;;            (+ ind ess-indent-level))
+          ;;           ((setq ind (ess--continued-statement containing-sexp))
+          ;;            (+ ind ess-continued-statement-offset))
+          ;;           (t 0))
+          ;; 0))
+          ((/= (char-after containing-sexp) ?{)
+           ;; line is expression, not statement:
+           ;; indent to just after the surrounding open.
+           (goto-char containing-sexp)
+           (let ((bol (line-beginning-position)))
+
+             (cond ((and (numberp ess-expression-offset)
+                         (re-search-backward "[ \t]*expression[ \t]*(" bol t))
+                    ;; obj <- expression(...
+                    ;; modified by shiba (forward-sexp -1)
+
+                    (beginning-of-line)
+                    (skip-chars-forward " \t")
+                    ;; end{modified}
+                    (+ (current-column) ess-expression-offset))
+                   ((and (numberp ess-arg-function-offset)
+                         (re-search-backward
+                          "=[ \t]*\\s\"*\\(\\w\\|\\s_\\)+\\s\"*[ \t]*"
+                          bol t))
+                    (forward-sexp -1)
+                    (+ (current-column) ess-arg-function-offset))
+                   ;; now, distinguish between
+                   ;;   a <- some.function(arg1,
+                   ;;                      arg2)
+                   ;; and
+                   ;;   a <- some.function(
+                   ;;     arg1,
+                   ;;     arg2)
+                   ;;
+                   ;; case 1: numeric
+                   ((and (numberp ess-arg-function-offset-new-line)
+                         (ess-looking-at-last-open-paren-p))
+                    (forward-sexp -1)
+                    (+ (current-column) ess-arg-function-offset-new-line))
+                   ;; case 2: list
+                   ((and (listp ess-arg-function-offset-new-line)
+                         (numberp (car ess-arg-function-offset-new-line))
+                         (ess-looking-at-last-open-paren-p))
+                    ;; flush args to the begining of
+                    (beginning-of-line)
+                    (skip-chars-forward " \t")
+                    (+ (current-column) (car ess-arg-function-offset-new-line)))
+                   ;; End
+                   (t
+                    (progn (goto-char (1+ containing-sexp))
+                           (current-column))))))
+          (t
+           ;; Statement level (containing-sexp char is "{").  Is it a continuation
+           ;; or a new statement? Find previous non-comment character.
+           (goto-char indent-point)
+           (ess-backward-to-noncomment containing-sexp)
+           ;; Back up over label lines, since they don't
+           ;; affect whether our line is a continuation.
+           (while (eq (preceding-char) ?\,)
+             (ess-backward-to-start-of-continued-exp containing-sexp)
+             (beginning-of-line)
+             (ess-backward-to-noncomment containing-sexp))
+           (forward-line 1)
+           ;; Now we get the answer.
+           (if (ess--continued-statement)
+               ;; This line is continuation of preceding line's statement;
+               ;; indent  ess-continued-statement-offset  more than the
+               ;; previous line of the statement.
+               (progn
+                 (ess-backward-to-start-of-continued-exp containing-sexp)
+                 (+ ess-continued-statement-offset (current-column)
+                    (if (save-excursion (goto-char indent-point)
+                                        (skip-chars-forward " \t")
+                                        (eq (following-char) ?{))
+                        ess-continued-brace-offset 0)))
+             ;; This line starts a new statement.
+             ;; Position following last unclosed open.
+             (goto-char containing-sexp)
+             ;; Is line first statement after an open-brace?
+             (or
+              ;; If no, find that first statement and indent like it.
+              (save-excursion
+                (forward-char 1)
+                (while (progn (skip-chars-forward " \t\n")
+                              (looking-at "#"))
+                  ;; Skip over comments following openbrace.
+                  (forward-line 1))
+                ;; The first following code counts
+                ;; if it is before the line we want to indent.
+                (and (< (point) indent-point)
+                     (current-column)))
+              ;; If no previous statement,
+              ;; indent it relative to line brace is on.
+              ;; For open brace in column zero, don't let statement
+              ;; start there too.  If ess-indent-level is zero,
+              ;; use ess-brace-offset +
+              ;; ess-continued-statement-offset instead.
+              ;; For open-braces not the first thing in a line,
+              ;; add in ess-brace-imaginary-offset.
+              (+ (if (and (bolp) (zerop ess-indent-level))
+                     (+ ess-brace-offset ess-continued-statement-offset)
+                   ess-indent-level)
+                 ;; Move back over whitespace before the openbrace.
+                 ;; If openbrace is not first nonwhite thing on the line,
+                 ;; add the ess-brace-imaginary-offset.
+                 (progn (skip-chars-backward " \t")
+                        (if (bolp) 0 ess-brace-imaginary-offset))
+                 ;; If the openbrace is preceded by a parenthesized exp,
+                 ;; move to the beginning of that;
+                 ;; possibly a different line
+                 (progn
+                   (if (eq (preceding-char) ?\))
+                       (forward-sexp -1))
+                   ;; Get initial indentation of the line we are on.
+                   (current-indentation)))))))))
+
+
 (defun ess-calculate-indent (&optional parse-start)
   "Return appropriate indentation for current line as ESS code.
 In usual case returns an integer: the column to indent to.
 Returns nil if line starts inside a string, t if in a comment."
   (save-excursion
     (beginning-of-line)
-    (let ((indent-point (point))
-          (beginning-of-defun-function nil) ;; don't call ess-beginning-of-function
-          (open-paren-in-column-0-is-defun-start t) ;; basically go to point-min (no better solution aparently)
-          (case-fold-search nil)
-          state
-          containing-sexp)
-      (if parse-start
-          (goto-char parse-start)
-        ;; this one is awful with open-paren-in-column-0-is-defun-start t, it always goes to point-min
-        (beginning-of-defun))
-      (while (< (point) indent-point)
-        (setq parse-start (point))
-        (setq state (parse-partial-sexp (point) indent-point 0))
-        (setq containing-sexp (car (cdr state))))
-      (cond ((or (nth 3 state) (nth 4 state))
-             ;; return nil (in string) or t (in comment)
-             (nth 4 state))
-            ((null containing-sexp)
-             ;; Line is at top level.  May be data or function definition
-             ;; or continuation of if,for,else not folowed by {
-             (beginning-of-line)
-             (let (orig-indent)
-               (if (and (/= (following-char) ?\{)
-                        (progn  (ess-backward-to-noncomment (point-min))
-                                (setq  orig-indent (ess-continued-statement-p))))
-                   (+ orig-indent ess-continued-statement-offset)
-                 0)))   ; Unless it starts a function body
-            ((/= (char-after containing-sexp) ?{)
-             ;; line is expression, not statement:
-             ;; indent to just after the surrounding open.
-             (goto-char containing-sexp)
-             (let ((bol (line-beginning-position)))
+    (cond
 
-               (cond ((and (numberp ess-expression-offset)
-                           (re-search-backward "[ \t]*expression[ \t]*(" bol t))
-                      ;; obj <- expression(...
-                      ;; modified by shiba (forward-sexp -1)
-                      (beginning-of-line)
-                      (skip-chars-forward " \t")
-                      ;; end{modified}
-                      (+ (current-column) ess-expression-offset))
-                     ((and (numberp ess-arg-function-offset)
-                           (re-search-backward
-                            "=[ \t]*\\s\"*\\(\\w\\|\\s_\\)+\\s\"*[ \t]*"
-                            bol t))
-                      (forward-sexp -1)
-                      (+ (current-column) ess-arg-function-offset))
-                     ;; now distinguish between
-                     ;;   a <- some.function(arg1,
-                     ;;                      arg2)
-                     ;; and
-                     ;;   a <- some.function(
-                     ;;     arg1,
-                     ;;     arg2)
-                     ;; case 1: numeric
-                     ((and (numberp ess-arg-function-offset-new-line)
-                           (looking-at "[ \t]*([ \t]*$"))
-                      (forward-sexp -1)
-                      (+ (current-column) ess-arg-function-offset-new-line))
-                     ;; case 2: list
-                     ((and (listp ess-arg-function-offset-new-line)
-                           (numberp (car ess-arg-function-offset-new-line))
-                           (looking-at "[ \t]*([ \t]*$"))
-                      ;; flush args to the begining of
-                      (beginning-of-line)
-                      (skip-chars-forward " \t")
-                      (+ (current-column) (car ess-arg-function-offset-new-line)))
-                     ;; "expression" is searched before "=".
-                     ;; End
-                     (t
-                      (progn (goto-char (1+ containing-sexp))
-                             (current-column))))))
-            (t
-             ;; Statement level.  Is it a continuation or a new statement?
-             ;; Find previous non-comment character.
-             (goto-char indent-point)
-             (ess-backward-to-noncomment containing-sexp)
-             ;; Back up over label lines, since they don't
-             ;; affect whether our line is a continuation.
-             (while (eq (preceding-char) ?\,)
-               (ess-backward-to-start-of-continued-exp containing-sexp)
-               (beginning-of-line)
-               (ess-backward-to-noncomment containing-sexp))
-             ;; Now we get the answer.
-             (if (ess-continued-statement-p)
-                 ;; This line is continuation of preceding line's statement;
-                 ;; indent  ess-continued-statement-offset  more than the
-                 ;; previous line of the statement.
-                 (progn
-                   (ess-backward-to-start-of-continued-exp containing-sexp)
-                   (+ ess-continued-statement-offset (current-column)
-                      (if (save-excursion (goto-char indent-point)
-                                          (skip-chars-forward " \t")
-                                          (eq (following-char) ?{))
-                          ess-continued-brace-offset 0)))
-               ;; This line starts a new statement.
-               ;; Position following last unclosed open.
-               (goto-char containing-sexp)
-               ;; Is line first statement after an open-brace?
-               (or
-                ;; If no, find that first statement and indent like it.
-                (save-excursion
-                  (forward-char 1)
-                  (while (progn (skip-chars-forward " \t\n")
-                                (looking-at "#"))
-                    ;; Skip over comments following openbrace.
-                    (forward-line 1))
-                  ;; The first following code counts
-                  ;; if it is before the line we want to indent.
-                  (and (< (point) indent-point)
-                       (current-column)))
-                ;; If no previous statement,
-                ;; indent it relative to line brace is on.
-                ;; For open brace in column zero, don't let statement
-                ;; start there too.  If ess-indent-level is zero,
-                ;; use ess-brace-offset +
-                ;; ess-continued-statement-offset instead.
-                ;; For open-braces not the first thing in a line,
-                ;; add in ess-brace-imaginary-offset.
-                (+ (if (and (bolp) (zerop ess-indent-level))
-                       (+ ess-brace-offset ess-continued-statement-offset)
-                     ess-indent-level)
-                   ;; Move back over whitespace before the openbrace.
-                   ;; If openbrace is not first nonwhite thing on the line,
-                   ;; add the ess-brace-imaginary-offset.
-                   (progn (skip-chars-backward " \t")
-                          (if (bolp) 0 ess-brace-imaginary-offset))
-                   ;; If the openbrace is preceded by a parenthesized exp,
-                   ;; move to the beginning of that;
-                   ;; possibly a different line
-                   (progn
-                     (if (eq (preceding-char) ?\))
-                         (forward-sexp -1))
-                     ;; Get initial indentation of the line we are on.
-                     (current-indentation))))))))))
+     ;; Identation for a closing parenthesis
+     ((looking-at "[[:blank:]]*\)")
+      (ess-calculate-indent--closing-paren))
 
-(defun ess-continued-statement-p ()
-  "If a continuatieon of a statement, return an indent to add to continuation."
-  (let ((eol (point)))
-    (save-excursion
-      (cond ((memq (preceding-char) '(nil ?\, ?\; ?\} ?\{ ?\]))
+     ;; Unindent lines that begin with ','
+     ((looking-at "[[:blank:]]*\,")
+      (let ((indent (save-excursion
+                      (ess-calculate-indent--default parse-start)))
+            (unindent (progn (skip-chars-forward " \t")
+                             ;; return number of skiped chars
+                             (skip-chars-forward ", \t"))))
+        (- indent unindent)))
+
+     ;; default indentation rules
+     (t
+      (ess-calculate-indent--default parse-start)))))
+
+
+(defun ess--continued-block-statement (&optional containing-sexp)
+  "If a continuation line of a block, return and indent of this line, otherwise nil."
+  (save-excursion
+    (beginning-of-line)
+    (ess-backward-to-noncomment containing-sexp)
+    (cond ((memq (preceding-char) '(nil ?\, ?\; ?\} ?\{ ?\] ?\())
+           nil)
+          ;; treat <- to avoid creating another check function
+          ((looking-back "<-")
+           (current-indentation))
+          ((= (preceding-char) ?\)) ;; if, for, while
+           (ignore-errors
+             ;; if throws an error clearly not a continuation
+             ;; can happen if the parenthetical statement starts a new line
+             ;; (foo)  ## or
+             ;; !(foo)
+             (forward-sexp -2)
+             (cond ((looking-at "for\\b[ \t]*(\\|while\\b[ \t]*(")
+                    (current-column))
+                   ((looking-at "if\\b[ \t]*(")
+                    (when (looking-back "\\belse[ \t]*")
+                      (backward-sexp 1))
+                    (current-column))
+                   ((looking-at "function\\b[ \t]*(")
+                    (current-indentation)))))
+          ((progn (ignore-errors (forward-sexp -1))
+                  (looking-at "else\\b\\|repeat\\b\\([:blank:]*\|\\&\\)"))
+           (let ((col (current-column)))
+             (skip-chars-backward " \t")
+             (if (or (bolp)
+                     (eq (preceding-char) ?\;))
+                 (- col (current-column))
+               (when (eq ?} (preceding-char))
+                 (- (current-column) 1)))))
+          )))
+
+
+(defun ess--continued-statement (&optional containing-sexp)
+  "If a continuation line, return an indent of this line, otherwise nil."
+  (save-excursion
+    (beginning-of-line)
+    (ess-backward-to-noncomment containing-sexp)
+    ;; this function is always called at eol
+    (let ((eol (point)))
+      (cond ((memq (preceding-char) '(nil ?\, ?\; ?\} ?\{ ?\] ?\())
              nil)
-            ;; ((bolp))
-            ((= (preceding-char) ?\))
-             (ignore-errors
-               ;; if throws an error clearly not a continuation
-               ;; can happen if the parenthetical statement starts a new line
-               ;; (foo)  ## or
-               ;; !(foo)
-               (forward-sexp -2)
-               (if (looking-at "if\\b[ \t]*(\\|for\\b[ \t]*(\\|while\\b[ \t]*(")
-                   (- (point) (point-at-bol))
-                 (when (looking-at "function\\b[ \t]*(")
-                   0))))
-            ((progn (forward-sexp -1)
-                    (and (looking-at "else\\b\\|repeat\\b")
-                         (not (looking-at "else\\s_\\|repeat\\s_"))))
-             (let ((pt (point)))
-               (skip-chars-backward " \t")
-               (if (or (bolp)
-                       (eq (preceding-char) ?\;))
-                   (- pt (point))
-                 (when (eq ?} (preceding-char))
-                   (- (point) (point-at-bol) 1)))))
-            (t
-             (when
-                 (progn (goto-char eol)
-                        (skip-chars-backward " \t")
-                        (or (and (> (current-column) 1)
-                                 (save-excursion (backward-char 1)
-                                                 (looking-at "[-:+*/><=&|]")))
-                            (and (> (current-column) 3)
-                                 (progn (backward-char 3)
-                                        (looking-at "%[^ \t]%")))))
-               0)
-             )))))
+            ((ignore-errors (up-list -1)
+                            (looking-back "if[ \t]*"))
+             ;; statements withing "if" condition it always ignore our
+             ;; continuation rules
+             nil)
+            ((progn (goto-char eol)
+                    (skip-chars-backward " \t")
+                    (or (and (> (current-column) 1)
+                             (and (not (looking-back "<-"))
+                                  (looking-back "[-:+*/><=&|]")))
+                        (and (> (current-column) 3)
+                             (progn (backward-char 3)
+                                    (looking-at "%[^ \t]%")))))
+             (let ((first-indent
+                    (or (and (/= ess-first-continued-statement-offset 0)
+                             (null (ess--continued-statement containing-sexp))
+                             ess-first-continued-statement-offset)
+                        0)))
+               (+ first-indent
+                  (if containing-sexp
+                      (if (> (point-at-bol) containing-sexp)
+                          (current-indentation)
+                        (goto-char containing-sexp)
+                        (1+ (current-column)))
+                    (+ (current-indentation))))))
+            (t nil)))))
 
 (defun ess-backward-to-noncomment (lim)
-  (let (opoint stop)
+  ;; this one is bad. Use
+  (let ((lim (or lim (point-min)))
+        opoint stop)
     (while (not stop)
       (skip-chars-backward " \t\n\f" lim)
       (setq opoint (point))
@@ -1333,7 +1308,8 @@ Returns nil if line starts inside a string, t if in a comment."
   (skip-chars-forward " \t"))
 
 (defun ess-backward-to-start-of-if (&optional limit)
-  "Move to the start of the last ``unbalanced'' if."
+  "Move to the start of the last ``unbalanced'' 'if' or 'else if'
+expression."
   (let ((beginning-of-defun-function nil))
     (or limit (setq limit (save-excursion (beginning-of-defun) (point))))
     (let ((if-level 1)
@@ -1343,6 +1319,8 @@ Returns nil if line starts inside a string, t if in a comment."
         (cond ((looking-at "else\\b")
                (setq if-level (1+ if-level)))
               ((looking-at "if\\b")
+               (when (looking-back "\\belse[[:blank:]]*")
+                 (backward-sexp 1))
                (setq if-level (1- if-level)))
               ((< (point) limit)
                (setq if-level 0)
@@ -1357,7 +1335,7 @@ style variables buffer local."
 
   (interactive)
   (let ((ess-styles (mapcar 'symbol-name (mapcar 'car ess-style-alist))))
-    (if (interactive-p)
+    (if (called-interactively-p 'any)
         (setq style
               (intern (ess-completing-read "Set ESS mode indentation style"
                                            ess-styles nil t nil nil ess-default-style))))
@@ -1368,7 +1346,7 @@ style variables buffer local."
       (error (format "Bad ESS style: %s" style)))
     (if (not quiet)
         (message "ESS-style: %s" ess-style))
-                                        ; finally, set the indentation style variables making each one local
+    ;; finally, set the indentation style variables making each one local
     (mapc (lambda (ess-style-pair)
             (make-local-variable (car ess-style-pair))
             (set (car ess-style-pair)
