@@ -20,9 +20,8 @@
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 ;;
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; A copy of the GNU General Public License is available at
+;; http://www.r-project.org/Licenses/
 
 ;;; Code:
 
@@ -1302,7 +1301,7 @@ Else
     (define-key sas-mode-local-map [return] 'newline-and-indent)
     (define-key sas-mode-local-map "\t" 'sas-indent-line)))
 
-(defvar ess-sas-edit-keys-toggle t
+(defvar ess-sas-edit-keys-toggle nil
   "Toggle TAB/RET key in `SAS-mode'.
 nil binds TAB to `sas-indent-line' and RET to `newline-and-indent'.
 Non-nil binds TAB to `ess-sas-tab-to-tab-stop',
