@@ -23,9 +23,9 @@
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; A copy of the GNU General Public License is available at
+;; http://www.r-project.org/Licenses/
+
 
 ;;; Commentary:
 
@@ -807,7 +807,7 @@ number."
     (adaptive-fill-mode           . nil)
     (indent-line-function         . 'sas-indent-line)
     ;;(indent-region-function       . 'sas-indent-region)
-    (require-final-newline        . t)
+    (require-final-newline        . mode-require-final-newline)
     (comment-start                . "/*")
     (comment-start-skip           . "/[*]")
     (comment-end                  . "*/")
@@ -1543,9 +1543,8 @@ page ;
 ;;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;;;    GNU General Public License for more details.
 ;;;
-;;;    You should have received a copy of the GNU General Public License
-;;;    along with this program; if not, write to the Free Software
-;;;    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+;;; A copy of the GNU General Public License is available at
+;;; http://www.r-project.org/Licenses/
 ;;;
 ;;;  Author:   Tom Cook
 ;;;            Dept. of Biostatistics
