@@ -21,9 +21,8 @@
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
-;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; A copy of the GNU General Public License is available at
+;; http://www.r-project.org/Licenses/
 
 ;;; Code:
 
@@ -36,10 +35,9 @@
   ;;FIXME (defun ess-calculate-indent ..)  can do that ...
   (interactive)
   (setq pos (or pos (point)))
-  (let ((pps (parse-partial-sexp (point-min) pos)))
-         ;; (if (featurep 'xemacs)
-                 ;;   (parse-partial-sexp (point-min) pos)
-                 ;; (syntax-ppss pos))))
+  (let ((pps
+         ;; (parse-partial-sexp (point-min) pos)
+         (syntax-ppss pos)))
     ;; 3: string,  4: comment
     (or (nth 3 pps) (nth 4 pps))))
 
@@ -47,14 +45,11 @@
 (defun ess-inside-string-p (&optional pos)
   "Return non-nil if point is inside string (according to syntax)."
   (interactive)
-  (setq pos (or pos (point)))
-  ;; next doesn't work reliably, when narrowing the buffer in iESS the ppss
-  ;; cahce is screwd :(
-  ;; (let ((pps (if (featurep 'xemacs)
-  ;;                (parse-partial-sexp (point-min) pos)
-  ;;              (syntax-ppss pos))))
-  ;;   (nth 3 pps))
-  (nth 3 (parse-partial-sexp (point-min) pos))
+  ;; when narrowing the buffer in iESS the ppss cahce is screwed:( But it is
+  ;; very fast, so don't bother for now.
+  (let ((pps (syntax-ppss pos)))
+    (nth 3 pps))
+  ;; (nth 3 (parse-partial-sexp (point-min) pos))
   )
 
 (defun ess-inside-comment-p (&optional pos)
@@ -67,14 +62,22 @@
 	    (eq 'font-lock-comment-face face)))
 	(nth 4 (parse-partial-sexp (progn (goto-char pos) (point-at-bol)) pos)))))
 
-(defun ess-inside-brackets-p (&optional pos)
+(defun ess-inside-brackets-p (&optional pos curly?)
   "Return t if position POS is inside brackets.
-POS defaults to point if no value is given."
+POS defaults to point if no value is given. If curly? is non nil
+also return t if inside curly brackets."
   (save-excursion
-    (let* ((pos (or pos (point)))
-	   (beg (re-search-backward "\\[" (max (point-min) (- pos 1000)) t))
-	   (end (re-search-forward "\\]" (min (point-max) (+ pos 1000)) t)))
-      (and beg end (> pos beg) (> end pos)))))
+    (let ((ppss (syntax-ppss pos))
+          (r nil))
+      (while (and (> (nth 0 ppss) 0)
+                  (not r))
+        (goto-char (nth 1 ppss))
+        (when (or (char-equal ?\[ (char-after))
+                  (and curly?
+                       (char-equal ?\{ (char-after))))
+          (setq r t))
+        (setq ppss (syntax-ppss)))
+      r)))
 
 (defun ess--extract-default-fl-keywords (keywords)
   "Extract the t-keywords from `ess-font-lock-keywords'."
@@ -112,7 +115,7 @@ POS defaults to point if no value is given."
             (buffer-list)))))
 
 
-(defun ess-generate-font-lock-submenu (menu)
+(defun ess--generate-font-lock-submenu (menu)
   "Internal, used to generate ESS font-lock submenu"
   (append (mapcar (lambda (el)
                     `[,(symbol-name (car el))
@@ -974,12 +977,12 @@ and y-offsets for the toolbar from point."
 ;; (defun ess-tooltip-show-at-point (text xo yo)
 ;;   (with-no-warnings
 ;;     (pos-tip-show text
-;;                   'popup-tip-face 
+;;                   'popup-tip-face
 ;;                   (point)
 ;;                   nil tooltip-hide-delay
 ;;                   popup-tip-max-width
 ;;                   nil xo yo)))
-      
+
 
 (defvar ess-build-tags-command nil
   "Command passed to generate tags.
@@ -988,7 +991,7 @@ If nil, `ess-build-tags-for-directory' uses the mode's imenu
 regexpresion. Othersiwe, it should be a string with two %s
 formats: one for directory and another for the output file.")
 
-  
+
 (defun ess-build-tags-for-directory (dir tagfile)
   "Ask for directory and tag file and build tags for current dialect.
 
@@ -1001,13 +1004,19 @@ Use M-. to navigate to a tag. M-x `visit-tags-table' to
 append/replace the currently used tag table.
 
 If prefix is given, force tag generation based on imenu. Might be
-useful when different languages are also present in the
-directory (.cpp, .c etc).
-"
-  (interactive "DDirectory to tag: 
-FTags file (default TAGS): ")
-  (when (eq (length (file-name-nondirectory tagfile)) 0)
-    (setq tagfile (concat tagfile "TAGS")))
+useful when different language files are also present in the
+directory (.cpp, .c etc)."
+  (interactive "DDirectory to tag:
+GTags file (default TAGS): ")
+  (when (or (eq (length (file-name-nondirectory tagfile)) 0)
+            (file-directory-p tagfile))
+    (setq tagfile (concat (file-name-as-directory tagfile) "TAGS")))
+  ;; emacs find-tags doesn't play well with remote TAG files :(
+  (when (file-remote-p tagfile)
+    (require 'tramp)
+    (setq tagfile (with-parsed-tramp-file-name tagfile foo foo-localname)))
+  (when (file-remote-p dir)
+    (setq dir (with-parsed-tramp-file-name dir foo foo-localname)))
   (if (and ess-build-tags-command (null current-prefix-arg))
       (ess-eval-linewise (format ess-build-tags-command dir tagfile))
     ;; else generate from imenu
@@ -1028,7 +1037,7 @@ FTags file (default TAGS): ")
       ;; (dbg (format "%s | %s" find-cmd tags-cmd))
       (when (= 0 (shell-command (format "%s | %s" find-cmd tags-cmd)))
         (message "Building tags .. ok!")))))
-      
+
 
 (defun ess-function-arguments (funname &optional proc)
   "Get FUNARGS from cache or ask the process for it.
@@ -1049,7 +1058,7 @@ is a special name that contains :,$ or @.
 If PROC is given, it should be an ESS process which should be
 queried for arguments.
 "
-  
+
   (when (and funname ;; usually returned by ess--funname.start (might be nil)
              (or proc (ess-process-live-p)))
     (let* ((proc (or proc (get-process ess-local-process-name)))
@@ -1097,36 +1106,43 @@ of lines.
 
 Also store the cons in 'ess--funname.start for potential use
 later."
-  (save-restriction
-    (let* ((proc (get-buffer-process (current-buffer)))
-           (mark (and proc (process-mark proc))))
+  (save-excursion
+    (save-restriction
+     (let* ((proc (get-buffer-process (current-buffer)))
+            (mark (and proc (process-mark proc))))
 
-      (if (and mark (>= (point) mark))
-          (narrow-to-region mark (point)))
+       (if (and mark (>= (point) mark))
+           (narrow-to-region mark (point)))
 
-      (and ess-noweb-mode
-           (ess-noweb-narrow-to-chunk))
+       (and ess-noweb-mode
+            (ess-noweb-narrow-to-chunk))
 
-      (when (not (ess-inside-string-p))
-        (setq ess--funname.start
-              (condition-case nil ;; check if it is inside a functon call
-                  (save-excursion
-                    (up-list -1)
-                    (while (not (looking-at "("))
-                      (up-list -1))
-                    ;; (skip-chars-backward " \t") ;; bad R style, so not providding help
-                    (let ((funname (symbol-name (symbol-at-point))))
-                      (when (and funname
-                                 (not (member funname ess-S-non-functions)))
-                        (cons funname (- (point) (length funname))))
-                      ))
-                (error nil)))))))
+       (unless (ess-inside-string-p)
+         (setq ess--funname.start
+               (condition-case nil ;; check if it is inside a functon 
+                   (progn
+                     ;; for the sake of big buffers, look only 1000 chars back
+                     (narrow-to-region (max (point-min) (- (point) 1000)) (point))
+                     (up-list -1)
+                     (while (not (looking-at "("))
+                       (up-list -1))
+                     (let ((funname (symbol-name (symbol-at-point))))
+                       (when (and funname
+                                  (not (member funname ess-S-non-functions)))
+                         (cons funname (- (point) (length funname))))
+                       ))
+                 (error nil))
+               ))))))
 
 (defun ess--inject-code-from-file (file)
   ;; this is different from ess-load-file
-  (ess-command (with-temp-buffer
-                 (insert-file-contents file)
-                 (buffer-string))))
+  (let ((content (with-temp-buffer
+                   (insert-file-contents file)
+                   (buffer-string))))
+    (when (string= ess-dialect "R")
+      ;; don't detect intermediate prompts
+      (setq content (concat "{" content "}\n")))
+    (ess-command content)))
 
 (provide 'ess-utils)
 
